import { Component, EventEmitter, HostListener, Inject, Injectable, Input, NgModule, Output, ViewChild } from '@angular/core';
import { MAT_DIALOG_DATA, MatButtonModule, MatDialog, MatDialogModule, MatDialogRef, MatGridListModule, MatIconModule, MatInputModule, MatListModule, MatProgressBarModule, MatSnackBarModule, MatToolbarModule, MatTooltipModule } from '@angular/material';
import { AsyncSubject, BehaviorSubject, Observable, from, of, throwError } from 'rxjs';
import { bufferCount, filter, last, map, mergeMap, take, tap } from 'rxjs/operators';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { HttpClient, HttpEventType, HttpHeaders, HttpRequest } from '@angular/common/http';
import { applyChange, diff } from 'deep-diff';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FileInfo = /** @class */ (function () {
    function FileInfo() {
    }
    return FileInfo;
}());
/**
 * @abstract
 */
var FileBrowserService = /** @class */ (function () {
    function FileBrowserService() {
        this.fileList = new BehaviorSubject([]);
        this.currentpath = new BehaviorSubject([]);
    }
    /**
     * @param {?} file
     * @return {?}
     */
    FileBrowserService.prototype.openFile = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        this.getDownloadObjectUrl(file.fullpath ? file.fullpath : file.name)
            .subscribe(function (url) {
            /** @type {?} */
            var a = document.createElement('a');
            a.href = url;
            a.download = file.name;
            a.style.display = 'none';
            a.target = '_blank';
            document.body.appendChild(a);
            a.click();
            document.body.removeChild(a);
            URL.revokeObjectURL(url);
        });
    };
    return FileBrowserService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var SimpleTextEditorDialogComponent = /** @class */ (function () {
    function SimpleTextEditorDialogComponent(dialogRef, data) {
        this.dialogRef = dialogRef;
        this.content = data.contents;
    }
    /**
     * @return {?}
     */
    SimpleTextEditorDialogComponent.prototype.cancel = /**
     * @return {?}
     */
    function () {
        this.dialogRef.close();
    };
    /**
     * @return {?}
     */
    SimpleTextEditorDialogComponent.prototype.save = /**
     * @return {?}
     */
    function () {
        this.dialogRef.close(this.content);
    };
    SimpleTextEditorDialogComponent.decorators = [
        { type: Component, args: [{
                    template: "<mat-dialog-content> <mat-form-field style=\"width: 100%\"> <textarea matInput [(ngModel)]=\"content\" matTextareaAutosize></textarea> </mat-form-field> </mat-dialog-content> <mat-dialog-actions> <button mat-icon-button (click)=\"cancel()\"> <mat-icon>cancel</mat-icon> </button> <button mat-icon-button (click)=\"save()\"> <mat-icon>done</mat-icon> </button> </mat-dialog-actions>"
                },] },
    ];
    /** @nocollapse */
    SimpleTextEditorDialogComponent.ctorParameters = function () { return [
        { type: MatDialogRef },
        { type: undefined, decorators: [{ type: Inject, args: [MAT_DIALOG_DATA,] }] }
    ]; };
    return SimpleTextEditorDialogComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
var FileChangeEventType = {
    RM_DIR: 0,
    RM_FILE: 1,
    SAVE_FILE: 2,
    MK_DIR: 3,
    RENAME: 4,
};
FileChangeEventType[FileChangeEventType.RM_DIR] = 'RM_DIR';
FileChangeEventType[FileChangeEventType.RM_FILE] = 'RM_FILE';
FileChangeEventType[FileChangeEventType.SAVE_FILE] = 'SAVE_FILE';
FileChangeEventType[FileChangeEventType.MK_DIR] = 'MK_DIR';
FileChangeEventType[FileChangeEventType.RENAME] = 'RENAME';
var FilesChangeEvent = /** @class */ (function () {
    function FilesChangeEvent(action, items) {
        this.action = action;
        this.items = items;
    }
    return FilesChangeEvent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/*
 *  Copyright 2010-2018 FinTech Neo AS ( fintechneo.com )- All rights reserved
 */
var FileBrowserComponent = /** @class */ (function () {
    function FileBrowserComponent(dialog, filebrowserservice) {
        this.dialog = dialog;
        this.filebrowserservice = filebrowserservice;
        this.dropText = 'Drop file here';
        this.fileschanges = new EventEmitter();
        this.showDropZone = false;
        this.draggingOverDropZone = false;
        this.uploadprogress = null;
        this.fileList = filebrowserservice.fileList;
    }
    /**
     * @return {?}
     */
    FileBrowserComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.fileInfoFilter) {
            this.fileList = this.fileList.pipe(map(function (fileList) {
                return fileList.filter(function (fileInfo) { return _this.fileInfoFilter(fileInfo); })
                    .sort(function (a, b) {
                    return ((a.isDir ? 'A' : 'B') + a.name).toLocaleLowerCase().localeCompare(((b.isDir ? 'A' : 'B') + b.name).toLocaleLowerCase());
                });
            }));
        }
    };
    /**
     * @return {?}
     */
    FileBrowserComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
    };
    /**
     * @return {?}
     */
    FileBrowserComponent.prototype.listExistingFiles = /**
     * @return {?}
     */
    function () {
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FileBrowserComponent.prototype.dragging = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        /** @type {?} */
        var dt = event.dataTransfer;
        if (dt.types) {
            /** @type {?} */
            var foundFilesType = false;
            for (var n = 0; n < dt.types.length; n++) {
                if (dt.types[n] === 'Files' || dt.types[n] === 'application/x-moz-file') {
                    foundFilesType = true;
                    break;
                }
            }
            if (foundFilesType) {
                event.preventDefault();
                this.showDropZone = true;
            }
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FileBrowserComponent.prototype.dragLeave = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.draggingOverDropZone = false;
        this.showDropZone = false;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FileBrowserComponent.prototype.invalidDrop = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        console.log('Drop outside valid drop target');
        event.preventDefault();
        this.hideDropZone();
    };
    /**
     * @return {?}
     */
    FileBrowserComponent.prototype.hideDropZone = /**
     * @return {?}
     */
    function () {
        this.draggingOverDropZone = false;
        this.showDropZone = false;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FileBrowserComponent.prototype.dropFiles = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        event.preventDefault();
        this.uploadFiles(event.dataTransfer.files);
        this.hideDropZone();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FileBrowserComponent.prototype.onFilesSelected = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.uploadFiles(event.target.files);
    };
    /**
     * @return {?}
     */
    FileBrowserComponent.prototype.attachFilesClicked = /**
     * @return {?}
     */
    function () {
        this.fileUploadInput.nativeElement.click();
    };
    /**
     * @param {?} file
     * @return {?}
     */
    FileBrowserComponent.prototype.deleteDir = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        var _this = this;
        this.filebrowserservice.rmdir(file.name).subscribe(function () {
            return _this.fileschanges.emit(new FilesChangeEvent(FileChangeEventType.RM_DIR, [file]));
        });
    };
    /**
     * @param {?} file
     * @return {?}
     */
    FileBrowserComponent.prototype.openFile = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        if (!this.fileActionsHandler || this.fileActionsHandler.openFile(file)) {
            this.filebrowserservice.openFile(file);
        }
    };
    /**
     * @param {?} file
     * @return {?}
     */
    FileBrowserComponent.prototype.deleteFile = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        var _this = this;
        if (!this.fileActionsHandler || this.fileActionsHandler.deleteFile(file)) {
            this.filebrowserservice.unlink(file.name).subscribe(function () {
                return _this.fileschanges.emit(new FilesChangeEvent(FileChangeEventType.RM_FILE, [file]));
            });
        }
    };
    /**
     * @param {?} files
     * @return {?}
     */
    FileBrowserComponent.prototype.uploadFiles = /**
     * @param {?} files
     * @return {?}
     */
    function (files) {
        var _this = this;
        /** @type {?} */
        var numfiles = files.length;
        from(Array.from(files).map(function (file) {
            return _this.fileActionsHandler ? _this.fileActionsHandler.beforeUpload(file)
                .pipe(mergeMap(function (proceed) {
                return proceed ? _this.filebrowserservice.uploadFile(file) : of(false);
            })) : _this.filebrowserservice.uploadFile(file);
        })).pipe(mergeMap(function (o) {
            return o.pipe(take(1));
        }, 1), bufferCount(numfiles), mergeMap(function () { return _this.filebrowserservice.readdir(); }))
            .subscribe(function () {
            return _this.fileschanges.emit(new FilesChangeEvent(FileChangeEventType.SAVE_FILE, Array.from(files)));
        });
    };
    /**
     * @return {?}
     */
    FileBrowserComponent.prototype.createFolder = /**
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var foldername = 'New folder';
        this.filebrowserservice.mkdir(foldername).subscribe(function () {
            return _this.fileschanges.emit(new FilesChangeEvent(FileChangeEventType.MK_DIR, foldername));
        });
    };
    /**
     * @param {?} file
     * @return {?}
     */
    FileBrowserComponent.prototype.rename = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        var _this = this;
        console.log('rename', this.newFileName);
        this.filebrowserservice.rename(file.name, this.newFileName).subscribe(function () {
            _this.fileschanges.emit(new FilesChangeEvent(FileChangeEventType.RENAME, [file, _this.newFileName]));
            _this.newFileName = null;
            _this.renameFile = null;
        });
    };
    /**
     * @param {?} file
     * @return {?}
     */
    FileBrowserComponent.prototype.changedir = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        this.filebrowserservice.changedir(file.name).subscribe();
    };
    /**
     *
     * @param indexfromcurrentpatharray (-1 = workdir root, otherwise 0 is first folder in current path array)
     */
    /**
     *
     * @param {?} indexfromcurrentpatharray (-1 = workdir root, otherwise 0 is first folder in current path array)
     * @return {?}
     */
    FileBrowserComponent.prototype.goToParent = /**
     *
     * @param {?} indexfromcurrentpatharray (-1 = workdir root, otherwise 0 is first folder in current path array)
     * @return {?}
     */
    function (indexfromcurrentpatharray) {
        var _this = this;
        this.filebrowserservice.currentpath
            .pipe(take(1), filter(function (p) { return p.length > indexfromcurrentpatharray + 1; }), mergeMap(function (p) {
            return _this.filebrowserservice.changedir(new Array(p.length - 1 - indexfromcurrentpatharray)
                .fill('..')
                .join('/'));
        }))
            .subscribe();
    };
    /**
     * @param {?} file
     * @return {?}
     */
    FileBrowserComponent.prototype.editFile = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        if (!this.fileActionsHandler || this.fileActionsHandler.editFile(file)) {
            this.openTextEditor(file);
        }
    };
    /**
     * @param {?} file
     * @return {?}
     */
    FileBrowserComponent.prototype.openTextEditor = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        var _this = this;
        this.filebrowserservice.getTextFileContents(file.name)
            .pipe(mergeMap(function (contents) {
            return _this.dialog.open(SimpleTextEditorDialogComponent, { data: { contents: contents } })
                .afterClosed();
        }), filter(function (res) { return res ? true : false; }), mergeMap(function (res) { return _this.filebrowserservice.saveTextFile(file.name, res); }), tap(function () { return _this.fileschanges.emit(new FilesChangeEvent(FileChangeEventType.SAVE_FILE, [file, _this.newFileName])); })).subscribe();
    };
    /**
     * @return {?}
     */
    FileBrowserComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        console.log('File browser component destroyed');
    };
    FileBrowserComponent.decorators = [
        { type: Component, args: [{
                    selector: 'app-filebrowser',
                    template: "<style> .dropzonehighlight { border: #155D97 dashed 5px; border-radius: 20px; background-color: rgba(255,255,255,0.8); text-align: center; } .overdropzone { border: #5D9715 solid 5px;         } mat-list-item .visibleOnHover { display: none; } mat-list-item:hover .visibleOnHover { display: inherit; } </style> <div style=\"display: flex; flex-direction: column; width: 100%; height: 100%\"> <div> <mat-toolbar style=\"width: 100%; display: flex;\">         <button mat-button (click)=\"goToParent(-1)\">Files</button>             <ng-container *ngFor=\"let dir of filebrowserservice.currentpath | async; let ndx=index\"> <mat-icon>chevron_right</mat-icon> <button mat-button (click)=\"goToParent(ndx)\">{{dir}}</button>             </ng-container> <span style=\"flex-grow: 1\"></span> <button mat-icon-button (click)=\"createFolder()\" matTooltip=\"New folder\"> <mat-icon>create_new_folder</mat-icon> </button> <button mat-icon-button (click)=\"attachFilesClicked()\" matTooltip=\"Upload file\"> <mat-icon>file_upload</mat-icon> </button> <input #attachmentFileUploadInput type=\"file\" [hidden]=\"true\" multiple (change)=\"onFilesSelected($event)\" /> </mat-toolbar> </div> <mat-progress-bar *ngIf=\"uploadprogress!=null\" style=\"width: 100%\" mode=\"determinate\" [value]=\"uploadprogress\" > </mat-progress-bar>         <div *ngIf=\"showDropZone\" style=\"flex-grow: 1\" [ngClass]=\"{'dropzonehighlight': showDropZone, 'overdropzone': draggingOverDropZone}\"  (dragover)=\"draggingOverDropZone=true\"  (dragleave)=\"draggingOverDropZone=false\"  (drop)=\"dropFiles($event)\"> <h1>Drop files here</h1>                               </div> <div style=\"overflow-y: auto; flex-grow: 1\" *ngIf=\"!showDropZone\"> <mat-nav-list> <ng-container *ngFor=\"let file of fileList | async\"> <ng-container *ngIf=\"file===renameFile\"> <mat-list-item> <mat-icon mat-list-icon *ngIf=\"file.isDir\">folder</mat-icon> <mat-form-field mat-line> <input matInput [(ngModel)]=\"newFileName\" /> </mat-form-field> <button mat-icon-button (click)=\"rename(file)\"> <mat-icon>done</mat-icon> </button> <button mat-icon-button (click)=\"renameFile=null\"> <mat-icon>cancel</mat-icon> </button> </mat-list-item> </ng-container> <ng-container *ngIf=\"file!==renameFile\"> <mat-list-item *ngIf=\"file.isDir && file.name!=='.' && file.name!=='..'\"> <mat-icon mat-list-icon *ngIf=\"file.isDir\">folder</mat-icon> <a (click)=\"changedir(file)\" mat-line>{{file.name}}</a>  <span class=\"visibleOnHover\"> <button mat-icon-button (click)=\"newFileName=file.name;renameFile=file\"> <mat-icon>edit</mat-icon> </button> <button mat-icon-button (click)=\"deleteDir(file)\"> <mat-icon>delete</mat-icon> </button> </span> </mat-list-item> <a mat-list-item (click)=\"changedir(file)\"  *ngIf=\" file.isDir && file.name==='..' && (filebrowserservice.currentpath | async).length > 0 \" > <mat-icon mat-list-icon *ngIf=\"file.isDir\">arrow_back</mat-icon> </a> <mat-list-item *ngIf=\"!file.isDir\" [ngClass]=\"{'filewithmergeconflict': file.conflict}\" > <a mat-line (click)=\"openFile(file)\">{{file.name}}</a>                         <span class=\"visibleOnHover\"> <button mat-icon-button (click)=\"editFile(file)\"> <mat-icon>edit</mat-icon> </button> <button mat-icon-button (click)=\"deleteFile(file)\"> <mat-icon>delete</mat-icon> </button> </span> </mat-list-item> </ng-container> </ng-container> </mat-nav-list>             </div> </div>",
                    styles: [".filewithmergeconflict { color:#F44336; }"]
                },] },
    ];
    /** @nocollapse */
    FileBrowserComponent.ctorParameters = function () { return [
        { type: MatDialog },
        { type: FileBrowserService }
    ]; };
    FileBrowserComponent.propDecorators = {
        fileInfoFilter: [{ type: Input }],
        fileActionsHandler: [{ type: Input }],
        fileschanges: [{ type: Output }],
        fileUploadInput: [{ type: ViewChild, args: ['attachmentFileUploadInput',] }],
        dragging: [{ type: HostListener, args: ['document:dragover', ['$event'],] }],
        dragLeave: [{ type: HostListener, args: ['document:dragleave', ['$event'],] }],
        invalidDrop: [{ type: HostListener, args: ['document:drop', ['$event'],] }]
    };
    return FileBrowserComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var SinglefolderviewComponent = /** @class */ (function () {
    function SinglefolderviewComponent(filebrowserservice) {
        this.filebrowserservice = filebrowserservice;
    }
    /**
     * @return {?}
     */
    SinglefolderviewComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.fileList = this.filebrowserservice.listendir(this.path);
    };
    SinglefolderviewComponent.decorators = [
        { type: Component, args: [{
                    selector: 'app-singlefolderview',
                    template: "<ng-container *ngFor=\"let file of fileList | async\"> <a mat-button (click)=\"filebrowserservice.openFile(file)\">{{file.name}}</a> </ng-container>",
                    styles: [""]
                },] },
    ];
    /** @nocollapse */
    SinglefolderviewComponent.ctorParameters = function () { return [
        { type: FileBrowserService }
    ]; };
    SinglefolderviewComponent.propDecorators = {
        path: [{ type: Input }]
    };
    return SinglefolderviewComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var GitProgressSnackbarComponent = /** @class */ (function () {
    function GitProgressSnackbarComponent() {
    }
    /**
     * @param {?} snackBar
     * @param {?} gitbackendservice
     * @return {?}
     */
    GitProgressSnackbarComponent.activate = /**
     * @param {?} snackBar
     * @param {?} gitbackendservice
     * @return {?}
     */
    function (snackBar, gitbackendservice) {
        /** @type {?} */
        var snackBarRef;
        gitbackendservice.currentStatus.subscribe(function (status) {
            if (status) {
                if (!snackBarRef) {
                    snackBarRef = snackBar.openFromComponent(GitProgressSnackbarComponent);
                }
                snackBarRef.instance.progressmessage = gitbackendservice.currentStatus;
            }
            else if (snackBarRef) {
                snackBarRef.dismiss();
                snackBarRef = null;
            }
        });
    };
    GitProgressSnackbarComponent.decorators = [
        { type: Component, args: [{
                    template: "{{progressmessage | async}}"
                },] },
    ];
    return GitProgressSnackbarComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
var ConflictPick = {
    MINE: 0,
    OLD: 1,
    YOURS: 2,
};
ConflictPick[ConflictPick.MINE] = 'MINE';
ConflictPick[ConflictPick.OLD] = 'OLD';
ConflictPick[ConflictPick.YOURS] = 'YOURS';
/**
 * @param {?} text
 * @param {?} pick
 * @return {?}
 */
function getJSONConflictVersion(text, pick) {
    /** @type {?} */
    var originalText = getConflictVersion(text, 1);
    if (pick === 1) {
        return originalText;
    }
    /** @type {?} */
    var original = JSON.parse(originalText);
    /** @type {?} */
    var mine = JSON.parse(getConflictVersion(text, 0));
    /** @type {?} */
    var yours = JSON.parse(getConflictVersion(text, 2));
    /** @type {?} */
    var minePatch = diff(original, mine);
    /** @type {?} */
    var yourPatch = diff(original, yours);
    if (pick === 0) {
        yourPatch.forEach(function (change) { return applyChange(original, null, change); });
        minePatch.forEach(function (change) { return applyChange(original, null, change); });
    }
    else if (pick === 2) {
        minePatch.forEach(function (change) { return applyChange(original, null, change); });
        yourPatch.forEach(function (change) { return applyChange(original, null, change); });
    }
    return JSON.stringify(original, null, 1);
}
/**
 * @param {?} text
 * @param {?} pick
 * @return {?}
 */
function getConflictVersion(text, pick) {
    while (hasConflicts(text)) {
        text = resolveNextConflict(text, pick);
    }
    return text;
}
/**
 * @param {?} text
 * @return {?}
 */
function hasConflicts(text) {
    /** @type {?} */
    var lines = text.split('\n');
    return lines.findIndex(function (l) { return l.startsWith('<<<<<<<'); }) > -1;
}
/**
 * @param {?} text
 * @param {?} pick
 * @return {?}
 */
function resolveNextConflict(text, pick) {
    /** @type {?} */
    var lines = text.split('\n');
    /** @type {?} */
    var conflictMineIndex = lines.findIndex(function (l) { return l.startsWith('<<<<<<<'); });
    /** @type {?} */
    var conflictOldIndex = lines.findIndex(function (l) { return l.startsWith('|||||||'); });
    /** @type {?} */
    var conflictYoursIndex = lines.findIndex(function (l) { return l.startsWith('======='); });
    /** @type {?} */
    var conflictEndIndex = lines.findIndex(function (l) { return l.startsWith('>>>>>>>'); });
    /** @type {?} */
    var chosen;
    switch (pick) {
        case ConflictPick.MINE:
            chosen = lines.slice(conflictMineIndex + 1, conflictOldIndex);
            break;
        case ConflictPick.OLD:
            chosen = lines.slice(conflictOldIndex + 1, conflictYoursIndex);
            break;
        case ConflictPick.YOURS:
            chosen = lines.slice(conflictYoursIndex + 1, conflictEndIndex);
            break;
    }
    return lines.slice(0, conflictMineIndex)
        .concat(chosen)
        .concat(lines.slice(conflictEndIndex + 1, lines.length)).join('\n');
}

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
var GIT_MERGE_FILE_FAVOR = {
    GIT_MERGE_FILE_FAVOR_NORMAL: 0,
    GIT_MERGE_FILE_FAVOR_OURS: 1,
    GIT_MERGE_FILE_FAVOR_THEIRS: 2,
    GIT_MERGE_FILE_FAVOR_UNION: 3,
};
GIT_MERGE_FILE_FAVOR[GIT_MERGE_FILE_FAVOR.GIT_MERGE_FILE_FAVOR_NORMAL] = 'GIT_MERGE_FILE_FAVOR_NORMAL';
GIT_MERGE_FILE_FAVOR[GIT_MERGE_FILE_FAVOR.GIT_MERGE_FILE_FAVOR_OURS] = 'GIT_MERGE_FILE_FAVOR_OURS';
GIT_MERGE_FILE_FAVOR[GIT_MERGE_FILE_FAVOR.GIT_MERGE_FILE_FAVOR_THEIRS] = 'GIT_MERGE_FILE_FAVOR_THEIRS';
GIT_MERGE_FILE_FAVOR[GIT_MERGE_FILE_FAVOR.GIT_MERGE_FILE_FAVOR_UNION] = 'GIT_MERGE_FILE_FAVOR_UNION';

/**
 * @param {?} buffer
 * @return {?}
 */
function hex(buffer) {
    /** @type {?} */
    var hexCodes = [];
    /** @type {?} */
    var view = new DataView(buffer);
    for (var i = 0; i < view.byteLength; i += 4) {
        // Using getUint32 reduces the number of iterations needed (we process 4 bytes each time)
        /** @type {?} */
        var value = view.getUint32(i);
        // toString(16) will give the hex representation of the number without padding
        /** @type {?} */
        var stringValue = value.toString(16);
        // We use concatenation and slice for padding
        /** @type {?} */
        var padding = '00000000';
        /** @type {?} */
        var paddedValue = (padding + stringValue).slice(-padding.length);
        hexCodes.push(paddedValue);
    }
    // Join all the hex strings into one
    return hexCodes.join('');
}
/**
 * @param {?} a
 * @param {?=} b
 * @return {?}
 */
function formatBytes(a, b) {
    if (0 === a) {
        return '0 B';
    }
    /** @type {?} */
    var c = 1e3;
    /** @type {?} */
    var d = b || 0;
    /** @type {?} */
    var e = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
    /** @type {?} */
    var f = Math.floor(Math.log(a) / Math.log(c));
    return parseFloat((a / Math.pow(c, f)).toFixed(d)) + ' ' + e[f];
}
var GitBackendService = /** @class */ (function (_super) {
    __extends(GitBackendService, _super);
    function GitBackendService(http$$1) {
        var _this = _super.call(this) || this;
        _this.http = http$$1;
        _this.workerReady = new AsyncSubject();
        _this.syncInProgress = false;
        _this.repositoryOpen = false;
        _this.workermessageid = 1;
        _this.resolvemap = {};
        _this.dirlisteners = {};
        _this.convertUploadsToLFSPointer = false;
        _this.gitLFSEndpoint = '/fintechneo/browsergittestdata.git/info';
        _this.gitLFSAuthorizationHeaderValue = 'Basic ' + btoa('username:password');
        _this.currentStatus = new BehaviorSubject(null);
        /**
         * Set to true to disable syncing filesystem with indexedDB
         */
        _this.syncLocalFSDisabled = false;
        return _this;
    }
    /**
     * @param {?} dir
     * @return {?}
     */
    GitBackendService.prototype.mount = /**
     * @param {?} dir
     * @return {?}
     */
    function (dir) {
        var _this = this;
        if (this.mountdir) {
            throwError('Mount already called');
        }
        this.currentStatus.next('Mounting local filesystem');
        this.mountdir = dir;
        // The "stupid" worker is simply evaluating scripts that we pass on to it
        this.worker = new Worker('assets/stupid_worker.js');
        this.worker.onmessage = function (msg) {
            if (msg.data.id) {
                if (msg.data.error) {
                    _this.resolvemap[msg.data.id]({ error: msg.data.error });
                }
                else {
                    _this.resolvemap[msg.data.id](msg.data.response);
                }
                delete _this.resolvemap[msg.data.id];
            }
            else if (msg.data.progressmessage) {
                _this.currentStatus.next(msg.data.progressmessage);
            }
        };
        // Set up emscripten with libgit2 (inside worker)
        return from(this.callWorker(function (params) {
            return new Promise(function (resolve, reject) {
                importScripts('libgit2.js');
                Module['onRuntimeInitialized'] = function () {
                    self.workdir = '/' + params.workdir;
                    FS.mkdir(self.workdir, '0777');
                    FS.mount(IDBFS, {}, self.workdir);
                    FS.chdir(self.workdir);
                    self.fromGitPath = function (path) {
                        /** @type {?} */
                        var currentdir = FS.cwd().split('/');
                        /** @type {?} */
                        var gitroot = self.workdir;
                        return gitroot + '/' + path;
                    };
                    self.toGitPath = function (filename) {
                        /** @type {?} */
                        var currentdir = FS.cwd().substring((self.workdir + "/").length).split('/');
                        return currentdir.join('/') + '/' + filename;
                    };
                    self.jsgitprogresscallback = function (msg) {
                        console.log(msg);
                        self.postMessage({
                            progressmessage: msg
                        });
                    };
                    self.jsgitinit();
                    resolve();
                };
            });
        }, { workdir: dir })).pipe(mergeMap(function () { return _this.syncLocalFS(true); }), mergeMap(function () { return (/** @type {?} */ (_this.readdir())); }), mergeMap(function (ret) {
            /** @type {?} */
            var hasGitRepo = ret.find(function (file) { return file.name === '.git'; });
            if (hasGitRepo) {
                return from(_this.callWorker(function () {
                    console.log('Open repository');
                    self.jsgitopenrepo();
                    self.jsgitsetuser(self.gituserfullname, self.gituseremail);
                    self.jsgitstatus();
                })).pipe(tap(function () { return _this.repositoryOpen = true; }), mergeMap(function () { return _this.readdir(); }));
            }
            else {
                return of(null);
            }
        }), tap(function () {
            _this.workerReady.next(true);
            _this.workerReady.complete();
            _this.currentStatus.next(null);
        }));
    };
    /**
     * Convert function to string source code and send it to the worker for evaluation
     * @param func the function to send
     * @param params an object that will be JSON stringified and passed to the function
     * @returns a promise that with the result of the operation
     */
    /**
     * Convert function to string source code and send it to the worker for evaluation
     * @param {?} func the function to send
     * @param {?=} params an object that will be JSON stringified and passed to the function
     * @return {?} a promise that with the result of the operation
     */
    GitBackendService.prototype.callWorker = /**
     * Convert function to string source code and send it to the worker for evaluation
     * @param {?} func the function to send
     * @param {?=} params an object that will be JSON stringified and passed to the function
     * @return {?} a promise that with the result of the operation
     */
    function (func, params) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            /** @type {?} */
            var workermessageid = _this.workermessageid++;
            _this.resolvemap[workermessageid] = function (result) {
                if (result && result.error) {
                    reject(result.error);
                }
                else {
                    resolve(result);
                }
            };
            _this.worker.postMessage({
                func: func.toString(),
                params: params,
                id: workermessageid
            });
        });
    };
    /**
     *
     * @param func the function to send to the worker
     * @param params params for the function to execute
     * @returns an Observable with the results (completes when done)
     */
    /**
     *
     * @param {?} func the function to send to the worker
     * @param {?=} params params for the function to execute
     * @return {?} an Observable with the results (completes when done)
     */
    GitBackendService.prototype.callWorker2 = /**
     *
     * @param {?} func the function to send to the worker
     * @param {?=} params params for the function to execute
     * @return {?} an Observable with the results (completes when done)
     */
    function (func, params) {
        var _this = this;
        return new Observable(function (observer) {
            /** @type {?} */
            var workermessageid = _this.workermessageid++;
            _this.resolvemap[workermessageid] = function (result) {
                if (result && result.error) {
                    observer.error(result.error);
                }
                else {
                    observer.next(result);
                }
                observer.complete();
            };
            _this.worker.postMessage({
                func: func.toString(),
                params: params,
                id: workermessageid
            });
        });
    };
    /**
     * @return {?}
     */
    GitBackendService.prototype.initRepository = /**
     * @return {?}
     */
    function () {
        return this.callWorker2(function () {
            self.jsgitinitrepo();
        });
    };
    /**
     * @param {?} url
     * @return {?}
     */
    GitBackendService.prototype.clone = /**
     * @param {?} url
     * @return {?}
     */
    function (url) {
        var _this = this;
        this.currentStatus.next('Cloning remote repository');
        return this.workerReady
            .pipe(mergeMap(function () {
            return _this.callWorker2(function (params) {
                self.jsgitclone(params.url, self.workdir);
                if (!self.jsgitgetlasterror()) {
                    self.jsgitstatusresult = [];
                    FS.chdir(self.workdir);
                    self.jsgitsetuser(self.gituserfullname, self.gituseremail);
                    self.jsgitgetlasterror();
                }
                return self.jsgitlasterrorresult;
            }, { url: url });
        }), map(function (err) {
            if (err && err.klass) {
                throw err;
            }
        }), mergeMap(function () { return _this.syncLocalFS(false); }), tap(function () { return _this.repositoryOpen = true; }), tap(function () { return _this.updateAllDirListeners(); }), tap(function () { return _this.currentStatus.next(null); }));
    };
    /**
     * @param path if not given, will read current directory and update listeners,
     * otherwise return a string array with the filenames of the given path.
     * @returns Observable with either string array of filenames if path is specified, or array of FileInfo if no path is specified
     */
    /**
     * @param {?=} path if not given, will read current directory and update listeners,
     * otherwise return a string array with the filenames of the given path.
     * @return {?} Observable with either string array of filenames if path is specified, or array of FileInfo if no path is specified
     */
    GitBackendService.prototype.readdir = /**
     * @param {?=} path if not given, will read current directory and update listeners,
     * otherwise return a string array with the filenames of the given path.
     * @return {?} Observable with either string array of filenames if path is specified, or array of FileInfo if no path is specified
     */
    function (path) {
        if (!path) {
            return this.readCurrentDirAndUpdateListeners();
        }
        return this._readdirString(path);
    };
    /*
     * simple readdir list the files names in the path and return as string array
     */
    /*
         * simple readdir list the files names in the path and return as string array
         */
    /**
     * @private
     * @param {?} path
     * @return {?}
     */
    GitBackendService.prototype._readdirString = /*
         * simple readdir list the files names in the path and return as string array
         */
    /**
     * @private
     * @param {?} path
     * @return {?}
     */
    function (path) {
        return this.callWorker2(function (params) {
            return FS.readdir(params.path);
        }, { path: path });
    };
    /*
     * Read contents from current directory and notify the fileList subject and single dir listeners
     */
    /*
         * Read contents from current directory and notify the fileList subject and single dir listeners
         */
    /**
     * @return {?}
     */
    GitBackendService.prototype.readCurrentDirAndUpdateListeners = /*
         * Read contents from current directory and notify the fileList subject and single dir listeners
         */
    /**
     * @return {?}
     */
    function () {
        var _this = this;
        return this.callWorker2(function () {
            return FS.readdir('.')
                .map(function (name) { return Object.assign({
                name: name,
                fullpath: FS.cwd() + '/' + name
            }, FS.stat(name)); })
                .map(function (fileinfo) {
                return Object.assign(fileinfo, { isDir: FS.isDir(fileinfo.mode) });
            });
        })
            .pipe(mergeMap(function (files) {
            return _this.repositoryOpen ?
                _this.listConflicts().pipe(map(function (cf) { return files.map(function (f) {
                    return Object.assign(f, {
                        conflict: cf.findIndex(function (c) {
                            return f.fullpath.substring(('/' + _this.mountdir + '/').length) === c;
                        }) > -1
                    });
                }); })) :
                of(files);
        }), tap(function (ret) {
            return _this.fileList.next(ret);
        }), tap(function () {
            return _this.currentpath.pipe(take(1), tap(function (p) { return _this.updateDirListener(p.join('/')); }));
        }));
    };
    /**
     * @param {?} name
     * @return {?}
     */
    GitBackendService.prototype.changedir = /**
     * @param {?} name
     * @return {?}
     */
    function (name) {
        var _this = this;
        return from(this.callWorker(function (params) { return FS.chdir(params.name); }, { name: name })).pipe(mergeMap(function () { return _this.readdir(); }), mergeMap(function () { return from(_this.callWorker(function () {
            return self.toGitPath('').split(/\//).filter(function (p) { return p.length > 0; });
        })); }), map(function (cwd) { return _this.currentpath.next(cwd); }));
    };
    /**
     * @param {?} filename
     * @return {?}
     */
    GitBackendService.prototype.unlink = /**
     * @param {?} filename
     * @return {?}
     */
    function (filename) {
        var _this = this;
        return from(this.callWorker(function (params) {
            FS.unlink(params.filename);
            console.log('Deleted local file', self.toGitPath(params.filename));
        }, { filename: filename })).pipe(mergeMap(function () { return _this.readdir(); }), mergeMap(function () { return _this.syncLocalFS(false); }));
    };
    /**
     * @return {?}
     */
    GitBackendService.prototype.listConflicts = /**
     * @return {?}
     */
    function () {
        return this.callWorker2(function () {
            return self.jsgitstatusresult.filter(function (r) { return r.status === 'conflict'; })
                .map(function (c) { return c.our; });
        });
    };
    /**
     * @return {?}
     */
    GitBackendService.prototype.listTextFileConflicts = /**
     * @return {?}
     */
    function () {
        return this.callWorker2(function () {
            return self.jsgitstatusresult.filter(function (r) {
                return r.status === 'conflict' &&
                    r.binary === 0;
            })
                .map(function (c) { return c.our; });
        });
    };
    /**
     * @param {?} fileName
     * @param {?} chooseConflictSide
     * @return {?}
     */
    GitBackendService.prototype.quickResolveConflict = /**
     * @param {?} fileName
     * @param {?} chooseConflictSide
     * @return {?}
     */
    function (fileName, chooseConflictSide) {
        var _this = this;
        return this.getTextFileContentsResolveConflicts(fileName, chooseConflictSide)
            .pipe(mergeMap(function (resolvedContents) {
            return _this.saveTextFile(fileName, resolvedContents);
        }));
    };
    /**
     * @return {?}
     */
    GitBackendService.prototype.markConflictsResolved = /**
     * @return {?}
     */
    function () {
        return this.callWorker2(function () {
            /** @type {?} */
            var conflictstatusresults = self.jsgitstatusresult.filter(function (c) { return c.status === 'conflict'; });
            conflictstatusresults.forEach(function (c) { return self.jsgitadd(c.our); });
            self.jsgitresolvemergecommit();
        });
    };
    /**
     * @param {?=} commitmessage
     * @return {?}
     */
    GitBackendService.prototype.commitChanges = /**
     * @param {?=} commitmessage
     * @return {?}
     */
    function (commitmessage) {
        var _this = this;
        return from(this.callWorker(function (params) {
            self.jsgitstatus();
            /** @type {?} */
            var noconflictstatusresults = self.jsgitstatusresult.filter(function (c) { return c.status !== 'conflict'; });
            /** @type {?} */
            var conflictstatusresults = self.jsgitstatusresult.filter(function (c) { return c.status === 'conflict'; });
            if (conflictstatusresults.length > 0) {
                console.log('Resolve conflicts before commiting changes.');
                return;
            }
            if (self.jsgitworkdirnumberofdeltas() > 0
                || noconflictstatusresults.length > 0) {
                noconflictstatusresults.forEach(function (p) {
                    if (p.status === 'deleted') {
                        self.jsgitremove(p.path);
                    }
                    else {
                        if (((/** @type {?} */ (p.path))).endsWith('/')) {
                            FS.readdir(self.fromGitPath(p.path))
                                .filter(function (f) { return f.charAt(0) !== '.'; })
                                .forEach(function (f) {
                                return self.jsgitadd(p.path + f);
                            });
                        }
                        self.jsgitadd(p.path);
                    }
                });
                self.jsgitaddfileswithchanges();
                /** @type {?} */
                var commitmessage_1 = params.commitmessage;
                if (!commitmessage_1) {
                    /** @type {?} */
                    var statusCount_1 = {};
                    noconflictstatusresults.forEach(function (r) {
                        if (!statusCount_1[r.status]) {
                            statusCount_1[r.status] = [];
                        }
                        statusCount_1[r.status].push(r.path);
                    });
                    commitmessage_1 = Object.keys(statusCount_1)
                        .map(function (statusName) { return statusName + " " + statusCount_1[statusName].length + " " +
                        ("(" + statusCount_1[statusName].join(',') + ")"); })
                        .join(', ');
                }
                self.jsgitcommit(commitmessage_1);
                console.log('Changes committed');
            }
            else {
                console.log('No changes');
            }
        }, { commitmessage: commitmessage })).pipe(mergeMap(function () { return _this.syncLocalFS(false); }));
    };
    /**
     * @param {?} filename
     * @return {?}
     */
    GitBackendService.prototype.getDownloadObjectUrl = /**
     * @param {?} filename
     * @return {?}
     */
    function (filename) {
        var _this = this;
        return this.callWorker2(function (params) {
            /** @type {?} */
            var contents = FS.readFile(params.filename);
            /** @type {?} */
            var lfsheader = "version https://git-lfs.github.com/spec/v1";
            /** @type {?} */
            var isLFSPointer = String.fromCharCode.apply(null, contents.subarray(0, lfsheader.length)) === lfsheader;
            if (isLFSPointer) {
                return FS.readFile(params.filename, { encoding: 'utf8' });
            }
            else {
                return URL.createObjectURL(new Blob([contents], { type: 'application/octet-stream' }));
            }
        }, { filename: filename }).pipe(mergeMap(function (url) {
            if (url.indexOf('version ') === 0) {
                /** @type {?} */
                var lfsPointerLines = url.split('\n');
                /** @type {?} */
                var filesize_1 = parseInt(lfsPointerLines[2].substring('size '.length), 10);
                return _this.getLFSDownloadInfo(url)
                    .pipe(mergeMap(function (downloadobj) {
                    /** @type {?} */
                    var downloadurl = _this.replaceWithProxyHost(downloadobj.href);
                    /** @type {?} */
                    var req = new HttpRequest('GET', downloadurl, {
                        headers: downloadobj.headers,
                        responseType: 'blob', reportProgress: true
                    });
                    return _this.http.request(req)
                        .pipe(map(function (event) {
                        if (event.type === HttpEventType.DownloadProgress) {
                            /** @type {?} */
                            var percentDone = Math.round(100 * event.loaded / filesize_1);
                            _this.currentStatus.next("Downloading " + formatBytes(event.loaded, 2) + " ( " + percentDone + " %)");
                        }
                        else if (event.type === HttpEventType.Response) {
                            return event.body;
                        }
                        return event;
                    }), last(), map(function (blob) {
                        return URL.createObjectURL(blob);
                    }), tap(function () { return _this.currentStatus.next(null); }));
                }));
            }
            else {
                return of(url);
            }
        }));
    };
    /**
     * @param {?} filename
     * @param {?} chosenConclictSide
     * @return {?}
     */
    GitBackendService.prototype.getTextFileContentsResolveConflicts = /**
     * @param {?} filename
     * @param {?} chosenConclictSide
     * @return {?}
     */
    function (filename, chosenConclictSide) {
        var _this = this;
        return this.getTextFileContents(filename)
            .pipe(mergeMap(function (contents) {
            if (hasConflicts(contents)) {
                /** @type {?} */
                var resolved_1 = filename.endsWith('.json') ?
                    getJSONConflictVersion(contents, chosenConclictSide) :
                    getConflictVersion(contents, chosenConclictSide);
                return _this.saveTextFile(filename, resolved_1)
                    .pipe(map(function () { return resolved_1; }));
            }
            else {
                return of(contents);
            }
        }));
    };
    /**
     * @param {?} filename
     * @return {?}
     */
    GitBackendService.prototype.getTextFileContents = /**
     * @param {?} filename
     * @return {?}
     */
    function (filename) {
        return from(this.callWorker(function (params) {
            return FS.readFile(params.filename, { encoding: 'utf8' });
        }, { filename: filename }));
    };
    /**
     * @param {?} filename
     * @param {?} content
     * @return {?}
     */
    GitBackendService.prototype.saveTextFile = /**
     * @param {?} filename
     * @param {?} content
     * @return {?}
     */
    function (filename, content) {
        var _this = this;
        return from(this.callWorker(function (params) {
            return FS.writeFile(params.filename, params.content);
        }, { filename: filename, content: content })).pipe(mergeMap(function () { return _this.syncLocalFS(false); }));
    };
    /**
     * @param {?} file
     * @return {?}
     */
    GitBackendService.prototype.uploadFile = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        var _this = this;
        if (this.convertUploadsToLFSPointer) {
            /** @type {?} */
            var filecontents_1;
            return new Observable(function (observer) {
                _this.currentStatus.next('Calculating checksum for ' + file.name);
                /** @type {?} */
                var filereader = new FileReader();
                filereader.readAsArrayBuffer(file);
                filereader.onload = function () { return __awaiter(_this, void 0, void 0, function () {
                    var hashbuf, lfsPointerText;
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                filecontents_1 = new Uint8Array((/** @type {?} */ (filereader.result)));
                                return [4 /*yield*/, window.crypto.subtle.digest('SHA-256', filecontents_1)];
                            case 1:
                                hashbuf = _a.sent();
                                lfsPointerText = "version https://git-lfs.github.com/spec/v1\n" +
                                    ("oid sha256:" + hex(hashbuf) + "\n") +
                                    ("size " + filecontents_1.length + "\n");
                                this.currentStatus.next("Uploading " + file.name + " to LFS storage");
                                observer.next(lfsPointerText);
                                return [2 /*return*/];
                        }
                    });
                }); };
            }).pipe(mergeMap(function (lfsPointerText) {
                return _this.uploadToLFSStorage(lfsPointerText, file).pipe(mergeMap(function () { return _this.saveTextFile(file.name, lfsPointerText); }));
            }), tap(function () { return _this.gitLFSTrack(file.name); }), tap(function () { return _this.currentStatus.next(null); }));
        }
        else {
            return new Observable(function (observer) {
                _this.callWorker(function (params) {
                    /** @type {?} */
                    var xhr = new XMLHttpRequest();
                    xhr.open('GET', params.url, false);
                    xhr.responseType = 'arraybuffer';
                    xhr.send();
                    console.log(xhr.response);
                    FS.writeFile(params.name, new Uint8Array(xhr.response), { encoding: 'binary' });
                    /** @type {?} */
                    var gitpath = self.toGitPath(params.name);
                    console.log('Written file locally', gitpath);
                }, { url: URL.createObjectURL(file), name: file.name })
                    .then(function () {
                    observer.next(file.name);
                });
            }).pipe(mergeMap(function () { return _this.syncLocalFS(false); }));
        }
    };
    /**
     * @param {?} dirname
     * @return {?}
     */
    GitBackendService.prototype.mkdir = /**
     * @param {?} dirname
     * @return {?}
     */
    function (dirname) {
        var _this = this;
        return from(this.callWorker(function (params) {
            FS.mkdir(params.dirname);
        }, { dirname: dirname })).pipe(mergeMap(function () { return _this.readdir(); }), mergeMap(function () { return _this.syncLocalFS(false); }));
    };
    /**
     * @param {?} dirname
     * @return {?}
     */
    GitBackendService.prototype.rmdir = /**
     * @param {?} dirname
     * @return {?}
     */
    function (dirname) {
        var _this = this;
        return this.callWorker2(function (params) {
            /** @type {?} */
            var dirnameparts = params.dirname.split('/');
            if (dirnameparts.length > 1) {
                FS.chdir('/' + dirnameparts.slice(0, dirnameparts.length - 1).join('/'));
                console.log('cwd', FS.cwd());
            }
            /** @type {?} */
            var recursedir = function (path) {
                return FS.readdir(path)
                    .filter(function (f) { return !(f === '.' || f === '..'); })
                    .forEach(function (f) {
                    /** @type {?} */
                    var deletepath = path + '/' + f;
                    console.log('Deleting', deletepath);
                    if (FS.isDir(FS.stat(deletepath).mode)) {
                        recursedir(deletepath);
                        FS.rmdir(deletepath);
                    }
                    else {
                        FS.unlink(deletepath);
                    }
                });
            };
            recursedir(params.dirname);
            FS.rmdir(params.dirname);
            console.log('Removed directory', params.dirname);
        }, { dirname: dirname }).pipe(mergeMap(function () { return _this.readdir(); }), mergeMap(function () { return _this.syncLocalFS(false); }));
    };
    /**
     * @param {?} oldpath
     * @param {?} newpath
     * @return {?}
     */
    GitBackendService.prototype.rename = /**
     * @param {?} oldpath
     * @param {?} newpath
     * @return {?}
     */
    function (oldpath, newpath) {
        var _this = this;
        return from(this.callWorker(function (params) {
            FS.rename(params.oldpath, params.newpath);
        }, { oldpath: oldpath, newpath: newpath })).pipe(mergeMap(function () { return _this.readdir(); }), mergeMap(function () { return _this.syncLocalFS(false); }));
    };
    /**
     * @return {?}
     */
    GitBackendService.prototype.push = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.currentStatus.next('Pushing local changes to server');
        return this.callWorker2(function () {
            self.jsgitpush();
            self.jsgitgetlasterror();
            return self.jsgitlasterrorresult;
        }, {}).pipe(map(function (err) {
            if (err && err.klass) {
                throw err;
            }
        }), tap(function () { return _this.currentStatus.next(null); }));
    };
    /**
     * @param {?=} git_merge_file_favor
     * @return {?}
     */
    GitBackendService.prototype.pull = /**
     * @param {?=} git_merge_file_favor
     * @return {?}
     */
    function (git_merge_file_favor) {
        var _this = this;
        if (git_merge_file_favor === void 0) { git_merge_file_favor = GIT_MERGE_FILE_FAVOR.GIT_MERGE_FILE_FAVOR_NORMAL; }
        this.currentStatus.next('Pulling recent changes from server');
        return this.callWorker2(function (params) {
            self.jsgitpull(params.git_merge_file_favor);
            self.jsgitgetlasterror();
            // Update status after pull
            self.jsgitstatus();
            return self.jsgitlasterrorresult;
        }, { git_merge_file_favor: git_merge_file_favor }).pipe(map(function (err) {
            if (err && err.klass) {
                throw err;
            }
        }), tap(function () { return _this.currentStatus.next(null); }), mergeMap(function () { return _this.readdir(); }), mergeMap(function () { return _this.syncLocalFS(false); }), tap(function () { return _this.updateAllDirListeners(); }));
    };
    /**
     * @param {?} dir
     * @return {?}
     */
    GitBackendService.prototype.listendir = /**
     * @param {?} dir
     * @return {?}
     */
    function (dir) {
        if (!this.dirlisteners[dir]) {
            this.dirlisteners[dir] = new BehaviorSubject([]);
            this.updateDirListener(dir);
        }
        return this.dirlisteners[dir];
    };
    /**
     * @return {?}
     */
    GitBackendService.prototype.updateAllDirListeners = /**
     * @return {?}
     */
    function () {
        var _this = this;
        Object.keys(this.dirlisteners).forEach(function (dir) { return _this.updateDirListener(dir); });
    };
    /**
     * @param {?} dir
     * @return {?}
     */
    GitBackendService.prototype.updateDirListener = /**
     * @param {?} dir
     * @return {?}
     */
    function (dir) {
        var _this = this;
        if (!this.dirlisteners[dir]) {
            return;
        }
        setTimeout(function () {
            return _this.workerReady.subscribe(function () {
                _this.callWorker(function (params) {
                    /** @type {?} */
                    var fulldirpath = self.fromGitPath(params.dir);
                    /** @type {?} */
                    var exists = false;
                    try {
                        FS.stat(fulldirpath);
                        exists = true;
                    }
                    catch (e) {
                    }
                    if (exists) {
                        console.log('update dir listener', fulldirpath, params.dir);
                        return FS.readdir(fulldirpath).map(function (name) { return Object.assign({
                            name: name,
                            fullpath: self.fromGitPath(params.dir + "/" + name)
                        }, FS.stat(self.fromGitPath(params.dir + "/" + name))); })
                            .map(function (fileinfo) {
                            return Object.assign(fileinfo, { isDir: FS.isDir(fileinfo.mode) });
                        }).filter(function (fileinfo) { return !fileinfo.isDir; });
                    }
                    else {
                        return [];
                    }
                }, { dir: dir }).then(function (files) {
                    console.log(dir, files);
                    _this.dirlisteners[dir].next(files);
                });
            });
        }, 0);
    };
    /**
     *
     * @param direction false = write, true = read
     */
    /**
     *
     * @param {?} direction false = write, true = read
     * @return {?}
     */
    GitBackendService.prototype.syncLocalFS = /**
     *
     * @param {?} direction false = write, true = read
     * @return {?}
     */
    function (direction) {
        var _this = this;
        console.log('synclocalfs called', direction, this.syncInProgress);
        if (this.syncLocalFSDisabled) {
            console.log('synclocalfs is disabled');
            return of(false);
        }
        if (this.syncInProgress) {
            return of(true);
        }
        else {
            this.syncInProgress = true;
            return new Observable(function (observer) {
                _this.callWorker(function (params) {
                    return new Promise(function (resolve, reject) {
                        return FS.syncfs(params.direction, function () {
                            console.log(params.direction ? 'reload persisted' : 'persisted');
                            resolve();
                        });
                    });
                }, { direction: direction }).then(function () {
                    _this.syncInProgress = false;
                    observer.next();
                });
            });
        }
    };
    /**
     * @return {?}
     */
    GitBackendService.prototype.unmount = /**
     * @return {?}
     */
    function () {
        if (this.worker) {
            this.worker.terminate();
            this.worker = null;
            console.log('Git backend service terminated');
            this.mountdir = null;
            this.repositoryOpen = false;
        }
    };
    /**
     * jsgit doesn't read the host name and protocol from the url so need to set it from here
     * @param host e.g. https://github.com
     */
    /**
     * jsgit doesn't read the host name and protocol from the url so need to set it from here
     * @param {?} host e.g. https://github.com
     * @return {?}
     */
    GitBackendService.prototype.setHost = /**
     * jsgit doesn't read the host name and protocol from the url so need to set it from here
     * @param {?} host e.g. https://github.com
     * @return {?}
     */
    function (host) {
        this.callWorker2(function (params) {
            return Module.jsgithost = params.host;
        }, { host: host }).subscribe();
    };
    /**
     * Set name and email to be used for commit
     * @param name
     * @param email
     */
    /**
     * Set name and email to be used for commit
     * @param {?} name
     * @param {?} email
     * @return {?}
     */
    GitBackendService.prototype.setUser = /**
     * Set name and email to be used for commit
     * @param {?} name
     * @param {?} email
     * @return {?}
     */
    function (name, email) {
        this.callWorker2(function (params) {
            self.gituserfullname = params.name;
            self.gituseremail = params.email;
            self.jsgitsetuser(self.gituserfullname, self.gituseremail);
        }, { name: name, email: email }).subscribe();
    };
    /**
     * Additional headers to send with http request, e.g. authorization tokens
     * @param headers
     */
    /**
     * Additional headers to send with http request, e.g. authorization tokens
     * @param {?} headers
     * @return {?}
     */
    GitBackendService.prototype.setHeaders = /**
     * Additional headers to send with http request, e.g. authorization tokens
     * @param {?} headers
     * @return {?}
     */
    function (headers) {
        this.callWorker2(function (params) {
            return Module.jsgitheaders = params.headers;
        }, {
            headers: headers.keys().map(function (key) {
                return ({ name: key, value: headers.get(key) });
            })
        }).subscribe();
    };
    /**
     * @return {?}
     */
    GitBackendService.prototype.history = /**
     * @return {?}
     */
    function () {
        return this.callWorker2(function () {
            self.jsgithistory();
            return self.jsgithistoryresult;
        });
    };
    /**
     * @param {?} url
     * @return {?}
     */
    GitBackendService.prototype.replaceWithProxyHost = /**
     * @param {?} url
     * @return {?}
     */
    function (url) {
        if (this.proxyHost) {
            url = url.substring('https://'.length);
            url = this.proxyHost + url.substring(url.indexOf('/'));
        }
        return url;
    };
    /**
     * Similar to e.g.: git lfs track "*.psd"
     *
     *  Will update .gitattributes
     * @param pattern
     */
    /**
     * Similar to e.g.: git lfs track "*.psd"
     *
     *  Will update .gitattributes
     * @param {?} filenameorpattern
     * @return {?}
     */
    GitBackendService.prototype.gitLFSTrack = /**
     * Similar to e.g.: git lfs track "*.psd"
     *
     *  Will update .gitattributes
     * @param {?} filenameorpattern
     * @return {?}
     */
    function (filenameorpattern) {
        this.callWorker2(function (params) {
            console.log('gitLFStrac', params);
            /** @type {?} */
            var pattern = params.pattern.replace(/\s/g, '[[:space:]]');
            /** @type {?} */
            var hasPattern = false;
            /** @type {?} */
            var gitattributeslines = [];
            try {
                gitattributeslines = FS.readFile('.gitattributes', { encoding: 'utf8' }).split('\n');
            }
            catch (e) { }
            /** @type {?} */
            var patternparts = pattern.split(/\./);
            /** @type {?} */
            var extension = patternparts[patternparts.length - 1];
            hasPattern = gitattributeslines.find(function (line) {
                /** @type {?} */
                var linepattern = line.split(' ')[0];
                /** @type {?} */
                var lineextension = linepattern.split('.')[1];
                return line.indexOf('filter=lfs') > 0 &&
                    line.indexOf(pattern + " ") === 0 ||
                    (line.charAt(0) === '*' && lineextension === extension);
            }) ? true : false;
            if (!hasPattern) {
                gitattributeslines.push(pattern + " filter=lfs diff=lfs merge=lfs -text");
                FS.writeFile('.gitattributes', gitattributeslines.join('\n'));
            }
            console.log(gitattributeslines);
        }, { pattern: filenameorpattern }).subscribe();
    };
    /**
     * @private
     * @param {?} lfsPointer
     * @param {?} contents
     * @return {?}
     */
    GitBackendService.prototype.uploadToLFSStorage = /**
     * @private
     * @param {?} lfsPointer
     * @param {?} contents
     * @return {?}
     */
    function (lfsPointer, contents) {
        var _this = this;
        /** @type {?} */
        var lfsPointerLines = lfsPointer.split('\n');
        /** @type {?} */
        var sha256sum = lfsPointerLines[1].substring('oid sha256:'.length);
        /** @type {?} */
        var size = parseInt(lfsPointerLines[2].substring('size '.length), 10);
        return this.http
            .post(this.gitLFSEndpoint + '/lfs/objects/batch', {
            'operation': 'upload',
            'transfers': ['basic'],
            // 'ref': { 'name': 'refs/heads/master' },
            'objects': [
                {
                    'oid': sha256sum,
                    'size': size,
                }
            ]
        }, {
            headers: {
                'Authorization': this.gitLFSAuthorizationHeaderValue,
                'Accept': 'application/vnd.git-lfs+json',
                'Content-Type': 'application/vnd.git-lfs+json'
            }
        }).pipe(tap(function (ret) { return console.log(ret); }), mergeMap(function (ret) {
            if (!ret.objects[0].actions || ret.objects[0].actions.download) {
                console.log('Already uploaded', sha256sum);
                return of(true);
            }
            /** @type {?} */
            var uploadobj = ret.objects[0].actions.upload;
            /** @type {?} */
            var verifyobj = ret.objects[0].actions.verify;
            /** @type {?} */
            var headers = new HttpHeaders(uploadobj.header);
            headers = headers.append('ngsw-bypass', 'true');
            /** @type {?} */
            var url = uploadobj.href;
            url = _this.replaceWithProxyHost(url);
            return _this.http.request(new HttpRequest('PUT', url, contents, { headers: headers, reportProgress: true }))
                .pipe(tap(function (event) {
                if (event.type === HttpEventType.UploadProgress) {
                    /** @type {?} */
                    var percentDone = Math.round(100 * event.loaded / event.total);
                    _this.currentStatus.next("Uploading " + formatBytes(event.loaded, 2) + " ( " + percentDone + " % )");
                }
                else if (event.type === HttpEventType.Response) {
                    return event.body;
                }
                return event;
            }), last(), mergeMap(function () { return verifyobj ?
                _this.http.post(_this.replaceWithProxyHost(verifyobj.href), {
                    oid: sha256sum,
                    size: size
                }, { headers: verifyobj.header })
                : of(true); }));
        }));
    };
    /**
     * @param {?} lfsPointer
     * @return {?}
     */
    GitBackendService.prototype.getLFSDownloadInfo = /**
     * @param {?} lfsPointer
     * @return {?}
     */
    function (lfsPointer) {
        /** @type {?} */
        var lfsPointerLines = lfsPointer.split('\n');
        /** @type {?} */
        var sha256sum = lfsPointerLines[1].substring('oid sha256:'.length);
        /** @type {?} */
        var size = parseInt(lfsPointerLines[2].substring('size '.length), 10);
        return this.http
            .post(this.gitLFSEndpoint + '/lfs/objects/batch', {
            'operation': 'download',
            'transfers': ['basic'],
            // 'ref': { 'name': 'refs/heads/master' },
            'objects': [
                {
                    'oid': sha256sum,
                    'size': size,
                }
            ]
        }, {
            headers: {
                'Authorization': this.gitLFSAuthorizationHeaderValue,
                'Accept': 'application/vnd.git-lfs+json',
                'Content-Type': 'application/vnd.git-lfs+json'
            }
        }).pipe(map(function (ret) {
            /** @type {?} */
            var downloadobj = ret.objects[0].actions.download;
            /** @type {?} */
            var url = downloadobj.href;
            /** @type {?} */
            var headers = null;
            if (downloadobj.header) {
                headers = new HttpHeaders(downloadobj.header);
            }
            return {
                href: url,
                headers: headers
            };
        }));
    };
    /**
     * @return {?}
     */
    GitBackendService.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.unmount();
    };
    GitBackendService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    GitBackendService.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    return GitBackendService;
}(FileBrowserService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FileBrowserModule = /** @class */ (function () {
    function FileBrowserModule() {
    }
    FileBrowserModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule,
                        FormsModule,
                        MatListModule,
                        MatButtonModule,
                        MatInputModule,
                        MatToolbarModule,
                        MatGridListModule,
                        MatSnackBarModule,
                        MatTooltipModule,
                        MatDialogModule,
                        MatIconModule,
                        MatProgressBarModule
                    ],
                    declarations: [
                        FileBrowserComponent,
                        SinglefolderviewComponent,
                        SimpleTextEditorDialogComponent,
                        GitProgressSnackbarComponent
                    ],
                    exports: [
                        FileBrowserComponent,
                        SinglefolderviewComponent
                    ],
                    entryComponents: [
                        SimpleTextEditorDialogComponent,
                        GitProgressSnackbarComponent
                    ]
                },] },
    ];
    return FileBrowserModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { GitProgressSnackbarComponent, FileBrowserComponent, FileBrowserService, FileInfo, GitBackendService, GIT_MERGE_FILE_FAVOR, FileBrowserModule, getJSONConflictVersion, getConflictVersion, hasConflicts, resolveNextConflict, ConflictPick, SimpleTextEditorDialogComponent as ɵb, SinglefolderviewComponent as ɵa };
