import { OnDestroy } from '@angular/core';
import { AsyncSubject, Observable, BehaviorSubject } from 'rxjs';
import { FileBrowserService } from '../filebrowser.service';
import { FileInfo } from '../filebrowser.service';
import { HttpHeaders, HttpClient } from '@angular/common/http';
import { ConflictPick } from './resolveconflict';
export declare enum GIT_MERGE_FILE_FAVOR {
    GIT_MERGE_FILE_FAVOR_NORMAL = 0,
    GIT_MERGE_FILE_FAVOR_OURS = 1,
    GIT_MERGE_FILE_FAVOR_THEIRS = 2,
    GIT_MERGE_FILE_FAVOR_UNION = 3
}
declare class LFSDownloadInfo {
    href: string;
    headers: HttpHeaders;
}
export declare class GitBackendService extends FileBrowserService implements OnDestroy {
    private http;
    workerReady: AsyncSubject<boolean>;
    worker: Worker;
    syncInProgress: boolean;
    repositoryOpen: boolean;
    mountdir: string;
    workermessageid: number;
    resolvemap: {
        [id: number]: Function;
    };
    dirlisteners: {
        [dir: string]: BehaviorSubject<FileInfo[]>;
    };
    convertUploadsToLFSPointer: boolean;
    gitLFSEndpoint: string;
    gitLFSAuthorizationHeaderValue: string;
    proxyHost: string;
    currentStatus: BehaviorSubject<string>;
    /**
     * Set to true to disable syncing filesystem with indexedDB
     */
    syncLocalFSDisabled: boolean;
    constructor(http: HttpClient);
    mount(dir: string): Observable<any>;
    /**
     * Convert function to string source code and send it to the worker for evaluation
     * @param func the function to send
     * @param params an object that will be JSON stringified and passed to the function
     * @returns a promise that with the result of the operation
     */
    callWorker(func: any, params?: any): Promise<any>;
    /**
     *
     * @param func the function to send to the worker
     * @param params params for the function to execute
     * @returns an Observable with the results (completes when done)
     */
    callWorker2(func: any, params?: any): Observable<any>;
    initRepository(): Observable<any>;
    clone(url: string): Observable<any>;
    /**
     * @param path if not given, will read current directory and update listeners,
     * otherwise return a string array with the filenames of the given path.
     * @returns Observable with either string array of filenames if path is specified, or array of FileInfo if no path is specified
     */
    readdir(path?: string): Observable<string[]> | Observable<FileInfo[]>;
    private _readdirString;
    readCurrentDirAndUpdateListeners(): Observable<FileInfo[]>;
    changedir(name: string): Observable<any>;
    unlink(filename: string): Observable<any>;
    listConflicts(): Observable<string[]>;
    listTextFileConflicts(): Observable<string[]>;
    quickResolveConflict(fileName: any, chooseConflictSide: ConflictPick): Observable<any>;
    markConflictsResolved(): Observable<any>;
    commitChanges(commitmessage?: string): Observable<any>;
    getDownloadObjectUrl(filename: string): Observable<string>;
    getTextFileContentsResolveConflicts(filename: string, chosenConclictSide: ConflictPick): Observable<string>;
    getTextFileContents(filename: string): Observable<string>;
    saveTextFile(filename: string, content: string): Observable<any>;
    uploadFile(file: File): Observable<any>;
    mkdir(dirname: string): Observable<any>;
    rmdir(dirname: string): Observable<any>;
    rename(oldpath: string, newpath: string): Observable<any>;
    push(): Observable<void>;
    pull(git_merge_file_favor?: number): Observable<any>;
    listendir(dir: string): Observable<FileInfo[]>;
    updateAllDirListeners(): void;
    updateDirListener(dir: string): void;
    /**
     *
     * @param direction false = write, true = read
     */
    syncLocalFS(direction: boolean): Observable<any>;
    unmount(): void;
    /**
     * jsgit doesn't read the host name and protocol from the url so need to set it from here
     * @param host e.g. https://github.com
     */
    setHost(host: string): void;
    /**
     * Set name and email to be used for commit
     * @param name
     * @param email
     */
    setUser(name: string, email: string): void;
    /**
     * Additional headers to send with http request, e.g. authorization tokens
     * @param headers
     */
    setHeaders(headers: HttpHeaders): void;
    history(): Observable<any>;
    replaceWithProxyHost(url: any): any;
    /**
     * Similar to e.g.: git lfs track "*.psd"
     *
     *  Will update .gitattributes
     * @param pattern
     */
    gitLFSTrack(filenameorpattern: string): void;
    private uploadToLFSStorage;
    getLFSDownloadInfo(lfsPointer: string): Observable<LFSDownloadInfo>;
    ngOnDestroy(): void;
}
export {};
